dofile_once("data/scripts/lib/utilities.lua")
local config = dofile_once("mods/MissileMinaMode/config.lua")

local entity_id = GetUpdatedEntityID()
local player_sprite_entity = EntityGetWithName("player_sprite")
local _, _, _, scale_x = EntityGetTransform(entity_id)
local x, y, rot = EntityGetTransform(player_sprite_entity)
if player_sprite_entity == 0 then
  return
end
local mouse_x, mouse_y = DEBUG_GetMouseWorld()

function get_direction( x1, y1, x2, y2 )
	return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
end

function move_angle_towards(current_angle, target_angle, speed)
  -- Make angle go from 0 to 2pi
  if target_angle < 0 then
    target_angle = target_angle + math.pi * 2
  end
  if current_angle < 0 then
    current_angle = current_angle + math.pi * 2
  end

  local dist_up = math.abs(target_angle - current_angle)
  local dist_down = math.abs((current_angle + math.pi * 2) - target_angle)
  if current_angle > target_angle then
    dist_up = math.abs((target_angle + math.pi * 2) - current_angle)
    dist_down = math.abs(current_angle - target_angle)
  end
  
  local dir
  if math.abs(dist_up) < math.abs(dist_down) then
    dir = 1
  else
    dir = -1
  end
  local dist_smallest = dist_up
  if dist_down < dist_up then
    dist_smallest = dist_down
  end

  local new_angle = current_angle + speed * dir
  if dir > 0 and current_angle < target_angle and new_angle > target_angle then
    new_angle = target_angle
  elseif dir < 0 and current_angle > target_angle and new_angle < target_angle  then
    new_angle = target_angle
  end
  new_angle = new_angle % (math.pi * 2)

  return new_angle
end

local dir = get_direction(x, y, mouse_x, mouse_y)
local current_angle = rot
local target_angle = dir
local new_angle = move_angle_towards(current_angle, target_angle, config.turning_speed)

local speed_multiplier = 1
boost_speed_bonus = boost_speed_bonus or 0
local throttle_fuel_depletion_amount = 0.05
local controls_component = EntityGetFirstComponent(entity_id, "ControlsComponent")
local character_data_component = EntityGetFirstComponent(entity_id, "CharacterDataComponent")
local character_platforming_component = EntityGetFirstComponent(entity_id, "CharacterPlatformingComponent")

local jetpack_active = GlobalsGetValue("mmm_jetpack_active", "false") == "true"

if GameHasFlagRun("movement_faster_picked") then
  speed_multiplier = speed_multiplier * 2
end

boost_speed_bonus = boost_speed_bonus + (jetpack_active and 1.5 or -2)
boost_speed_bonus = math.max(0, math.min(boost_speed_bonus, 50))

local vx = math.cos(new_angle) * (config.flying_speed + boost_speed_bonus) * speed_multiplier
local vy = math.sin(new_angle) * (config.flying_speed + boost_speed_bonus) * speed_multiplier

local sprite_component = EntityGetFirstComponent(player_sprite_entity, "SpriteComponent")
ComponentSetValue(sprite_component, "has_special_scale", scale_x > 0 and "0" or "1")

-- Are we being teleported? Then just stop flying.
if GlobalsGetValue("mmm_teleport_start_x", "") ~= "" then
  vx = 0
  vy = 0
end

-- This is necessary to counteract some weird gravity stuff that we can't disable?
--vy = vy - 5.9

EntitySetTransform(player_sprite_entity, x, y, new_angle)
ComponentSetValue2(character_data_component, "mVelocity", vx, vy)

function polar_to_carthesian(angle, length)
  return math.cos(angle) * length, math.sin(angle) * length
end

function carthesian_to_polar(x, y)
  return get_direction(0, 0, x, y), get_magnitude(x, y)
end

local hand_pos_polar_angle, hand_pos_polar_length = carthesian_to_polar(7, -3 * scale_x)
local cape_pos_polar_angle, cape_pos_polar_length = carthesian_to_polar(3 * scale_x, -2 * scale_x)

local hotspot_hand = EntityGetFirstComponent(entity_id, "HotspotComponent", "hand")
local hotspot_cape = EntityGetFirstComponent(entity_id, "HotspotComponent", "cape")

ComponentSetValueVector2(hotspot_hand, "offset", polar_to_carthesian(hand_pos_polar_angle + rot, hand_pos_polar_length))
ComponentSetValueVector2(hotspot_cape, "offset", polar_to_carthesian(cape_pos_polar_angle + (rot * scale_x), cape_pos_polar_length))
